/*********************************************************************************
*                    LED-Matrix-8x8
*                   ================
*                   Uwe Berger; 2009
*
* Template als Grundlage fuer Applikationen zu einem einfachen ATmega8-
* Board, an dem eine 8x8-LED-Matrix und drei Taster angeschlossen sind.
* 
* Es sind zwei Modi implementiert, die via Define TWO_BIT_DOTS gesteuert
* werden koennen:
*  -Define nicht gesetzt --> LED an/aus
*  -Define gesetzt --> Versuch die LEDs in 4 Helligkeitsstufen (2Bit)
* 	                   anzusteuern   
* 
* Die Taster sind in diesem Template noch nicht entprellt. Eine der
* Taster ist als ein-/Ausschalter vorgesehen.
* 
* ---------
* Have fun!
*
**********************************************************************************/
#ifndef F_CPU
	#define F_CPU 1000000
#endif

#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

// wenn 2Bit-"Graustufen", dann folgendes Define auskommentieren,
// ansonsten "Schwarz/weiss"
//#define TWO_BIT_DOTS 


//*** Hardwaredefinitionen ***
// Matrix-Zeilen
#define ROW_0_DDR		DDRC
#define ROW_0_PORT		PORTC
#define ROW_0			PC0
#define ROW_1_DDR		DDRC
#define ROW_1_PORT		PORTC
#define ROW_1			PC1
#define ROW_2_DDR		DDRC
#define ROW_2_PORT		PORTC
#define ROW_2			PC2
#define ROW_3_DDR		DDRC
#define ROW_3_PORT		PORTC
#define ROW_3			PC3
#define ROW_4_DDR		DDRC
#define ROW_4_PORT		PORTC
#define ROW_4			PC4
#define ROW_5_DDR		DDRC
#define ROW_5_PORT		PORTC
#define ROW_5			PC5
#define ROW_6_DDR		DDRD
#define ROW_6_PORT		PORTD
#define ROW_6			PD0
#define ROW_7_DDR		DDRD
#define ROW_7_PORT		PORTD
#define ROW_7			PD1
// Matrix-Spalten
#define COL_0_DDR		DDRD
#define COL_0_PORT		PORTD
#define COL_0			PD3
#define COL_1_DDR		DDRD
#define COL_1_PORT		PORTD
#define COL_1			PD4
#define COL_2_DDR		DDRD
#define COL_2_PORT		PORTD
#define COL_2			PD5
#define COL_3_DDR		DDRD
#define COL_3_PORT		PORTD
#define COL_3			PD6
#define COL_4_DDR		DDRD
#define COL_4_PORT		PORTD
#define COL_4			PD7
#define COL_5_DDR		DDRB
#define COL_5_PORT		PORTB
#define COL_5			PB0
#define COL_6_DDR		DDRB
#define COL_6_PORT		PORTB
#define COL_6			PB1
#define COL_7_DDR		DDRB
#define COL_7_PORT		PORTB
#define COL_7			PB2

// Taster
#define TASTER_ON_OFF_DDR	DDRD
#define TASTER_ON_OFF_PORT	PORTD
#define TASTER_ON_OFF_PINX  PIND
#define TASTER_ON_OFF_PIN	PD2

#define TASTER_LEFT_DDR		DDRB
#define TASTER_LEFT_PORT	PORTB
#define TASTER_LEFT_PINX	PINB
#define TASTER_LEFT_PIN		PB6

#define TASTER_RIGHT_DDR	DDRB
#define TASTER_RIGHT_PORT	PORTB
#define TASTER_RIGHT_PINX	PINB
#define TASTER_RIGHT_PIN	PB7

// Tastencodes
#define ON_OFF	1
#define LEFT	2
#define RIGHT	3

volatile uint8_t row = 0;

#ifdef TWO_BIT_DOTS
	#define PRELOAD_T0 238
	volatile uint8_t dimm_count = 0;
	volatile uint16_t led[8] = {
								0b0000000000000000,
								0b0101010101010101,
								0b1010101010101010,
								0b1111111111111111,
								0b1111111111111111,
								0b1010101010101010,
								0b0101010101010101,
								0b0000000000000000
								};
#else
	#define PRELOAD_T0 217
	volatile uint8_t led[8] = {
								0b10101010,
								0b01010101,
								0b10101010,
								0b01010101,
								0b10101010,
								0b01010101,
								0b10101010,
								0b01010101
								};
#endif


void matrix_clear(void) {
	// alle Zeilen aus (Low)
	ROW_0_PORT &= ~(1<<ROW_0);
	ROW_1_PORT &= ~(1<<ROW_1);
	ROW_2_PORT &= ~(1<<ROW_2);
	ROW_3_PORT &= ~(1<<ROW_3);
	ROW_4_PORT &= ~(1<<ROW_4);
	ROW_5_PORT &= ~(1<<ROW_5);
	ROW_6_PORT &= ~(1<<ROW_6);
	ROW_7_PORT &= ~(1<<ROW_7);
	
	// alle Spalten aus (High)
	COL_0_PORT |= (1<<COL_0);
	COL_1_PORT |= (1<<COL_1);
	COL_2_PORT |= (1<<COL_2);
	COL_3_PORT |= (1<<COL_3);
	COL_4_PORT |= (1<<COL_4);
	COL_5_PORT |= (1<<COL_5);
	COL_6_PORT |= (1<<COL_6);
	COL_7_PORT |= (1<<COL_7);
}


//************************************************************************
ISR(TIMER0_OVF_vect)
{
	TCNT0 = PRELOAD_T0;	

	// Matrix zuruecksetzen
	matrix_clear();
		
	// naechste Zeile/Spalte ermitteln
	row++;
	if (row > 7) row = 0;
	
	// Spalten setzen
#ifdef TWO_BIT_DOTS	
	// dimm_counter steuert die "Austastluecken"...
	dimm_count++;
	if (dimm_count > 2) dimm_count = 0;
	// ...es werden 2 Bits aus dem Integer maskiert und das Ergebnis
	// "rechts-buendig" auf die 0. Stelle geschoben...
	if (((led[row]&49152)/16384)	> dimm_count) COL_0_PORT &= ~(1<<COL_0);
	if (((led[row]&12288)/4096)		> dimm_count) COL_1_PORT &= ~(1<<COL_1);
	if (((led[row]&3072)/1024)		> dimm_count) COL_2_PORT &= ~(1<<COL_2);
	if (((led[row]&768)/256)		> dimm_count) COL_3_PORT &= ~(1<<COL_3);
	if (((led[row]&192)/64)			> dimm_count) COL_4_PORT &= ~(1<<COL_4);
	if (((led[row]&48)/16)			> dimm_count) COL_5_PORT &= ~(1<<COL_5);
	if (((led[row]&12)/4)			> dimm_count) COL_6_PORT &= ~(1<<COL_6);
	if (((led[row]&3)/1)			> dimm_count) COL_7_PORT &= ~(1<<COL_7);
#else
	if (led[row]&0b10000000) COL_0_PORT &= ~(1<<COL_0);
	if (led[row]&0b01000000) COL_1_PORT &= ~(1<<COL_1);
	if (led[row]&0b00100000) COL_2_PORT &= ~(1<<COL_2);
	if (led[row]&0b00010000) COL_3_PORT &= ~(1<<COL_3);
	if (led[row]&0b00001000) COL_4_PORT &= ~(1<<COL_4);
	if (led[row]&0b00000100) COL_5_PORT &= ~(1<<COL_5);
	if (led[row]&0b00000010) COL_6_PORT &= ~(1<<COL_6);
	if (led[row]&0b00000001) COL_7_PORT &= ~(1<<COL_7);
#endif
	
	// naechste Zeile ein (Low)
	switch (row) {
		case 0: ROW_0_PORT |= (1 << ROW_0); break;
		case 1: ROW_1_PORT |= (1 << ROW_1); break;
		case 2: ROW_2_PORT |= (1 << ROW_2); break;
		case 3: ROW_3_PORT |= (1 << ROW_3); break;
		case 4: ROW_4_PORT |= (1 << ROW_4); break;
		case 5: ROW_5_PORT |= (1 << ROW_5); break;
		case 6: ROW_6_PORT |= (1 << ROW_6); break;
		case 7: ROW_7_PORT |= (1 << ROW_7); break;
	}
}

//************************************************************************
ISR(INT0_vect)
{
	// deklariert, um wieder aus Sleep-Mode zu kommen
}

//************************************************************************
void my_delay_ms(uint16_t ms)
{
	uint16_t i;
	for (i=0; i<ms; i++) _delay_ms(1);
}



//************************************************************************
uint8_t getkey(void) 
{
	if (bit_is_clear(TASTER_ON_OFF_PINX, TASTER_ON_OFF_PIN))	return ON_OFF;
	if (bit_is_clear(TASTER_LEFT_PINX, TASTER_LEFT_PIN))		return LEFT;
	if (bit_is_clear(TASTER_RIGHT_PINX, TASTER_RIGHT_PIN))		return RIGHT;
	return 0;
}

//************************************************************************
//************************************************************************
//************************************************************************
int main(void)
{
	
	// Hardware initialisieren...
	// ... Analogcomperator ausschalten (Strom sparen!)
	ACSR = 0x80;		

	//... Ausgaenge
	ROW_0_DDR |= (1<<ROW_0);
	ROW_1_DDR |= (1<<ROW_1);
	ROW_2_DDR |= (1<<ROW_2);
	ROW_3_DDR |= (1<<ROW_3);
	ROW_4_DDR |= (1<<ROW_4);
	ROW_5_DDR |= (1<<ROW_5);
	ROW_6_DDR |= (1<<ROW_6);
	ROW_7_DDR |= (1<<ROW_7);
	COL_0_DDR |= (1<<COL_0);
	COL_1_DDR |= (1<<COL_1);
	COL_2_DDR |= (1<<COL_2);
	COL_3_DDR |= (1<<COL_3);
	COL_4_DDR |= (1<<COL_4);
	COL_5_DDR |= (1<<COL_5);
	COL_6_DDR |= (1<<COL_6);
	COL_7_DDR |= (1<<COL_7);
	
	// ... Eingaenge
	TASTER_ON_OFF_PORT &= ~(1 << TASTER_ON_OFF_PIN);
	TASTER_LEFT_PORT   &= ~(1 << TASTER_LEFT_PIN);
	TASTER_RIGHT_PORT  &= ~(1 << TASTER_RIGHT_PIN);
	
	// ... Timer initialisieren
    TCNT0   = PRELOAD_T0;							// Preloadwert Timer0 setzen
	TCCR0 |= (1 << CS01)|(1 << CS00);				// 64
	TIMSK  |= (1 << TOIE0);							// Timer0-OVF einschalten
	// ... externe Interrupts
	//GICR  |= (1<<INT0);          					// externer Interrupt an INT0
	// ... Interrupts einschalten
	sei();
		
	
	while (1){
		// Tasten abfragen/reagieren
		switch (getkey()) {
			case ON_OFF:	{	// **** On/Off-Schalter ****
								// Timer0-OVF ausschalten
								TIMSK  &= ~(1 << TOIE0);
								// Ports entsprechend "loeschen"...
								matrix_clear();
								// INT0 einschalten
								GICR  |= (1<<INT0);
								// Sleep-Mode einstellen
								set_sleep_mode(SLEEP_MODE_PWR_DOWN);	
								// Sleep-Mode einschalten
								sleep_mode();
								// INT0 ausschalten		
								GICR  &= ~(1<<INT0);
								//	Timer0-OVF einschalten				
								TIMSK  |= (1 << TOIE0);
								break;
							}
			case LEFT:		{
								break;
							}
			case RIGHT:		{
								break;
							}
		}
	}
}
